
//	Save the formatter to the prism object for later use
if (!prism.formatter) {
	//	Get the filter
	var filter = prism.$injector.get('$filter');
	//	Create a global
	prism.formatter = {
		// 	Usage: prism.formatter.numeric(value,mask);
		numeric: filter('numeric'),	
		//	Usage: prism.formatter.date(dateObj,mask); OR prism.formatter.date(stringDate,mask);
		datetime: filter('levelDate')	
	};
}

//	Initialize the new widget
prism.registerWidget("customHighchartWidget", {
	name : "customHighchartWidget",
	family : "Line",
	title : "customHighchartWidget",
	iconSmall : "/plugins/customHighchartWidget/icon-24.png",
	styleEditorTemplate: "/plugins/customHighchartWidget/styler.html",
	style: {
		showLegend: false,
		showXAxisLabel: false,
		chartType: 'line',		//	Can be 'line', 'area'
		showStep: true,			//	set as True to show boxed lines 
		showMarkers: false,
		groupTooltips: true
	},
	data : {
		selection : [],
		defaultQueryResult : {},	
		panels : [
			{
				name: 'X-Axis',
				type: "visible",
				metadata: {
					types: ['dimensions'],
					maxitems: 1
				},
				allowedDataTypes: ["datetime","numeric"],
				visibility: true
			},
			{
				name: 'value',
				type: 'visible',	
				itemAttributes: ["color"],
				allowedColoringTypes: function() {
					return  {
						color: true,
						condition: false,
						range: false
					};
				},
				metadata: {
					types: ['measures'],
					maxitems: -1
				},
				itemAdded: function(widget, item) {
					var colorFormatType = $$get(item, "format.color.type");
					if ("color" === colorFormatType || "color" === colorFormatType) {
						var color = item.format.color.color;
						defined(color) && "transparent" != color && "white" != color && "#fff" != color && "#ffffff" != color || $jaql.resetColor(item)
					}
					"range" === colorFormatType && $jaql.resetColor(item), defined(item, "format.color_bkp") && $jaql.resetColor(item), defined(item, "format.members") && delete item.format.members
				}
			},
			{
				name: 'breakby',
				label: "Break By",
				type: "series",
				itemAttributes: ["color"],
				allowedColoringTypes: function() {
					return {
						color: true,
						condition: false,
						range: false
					}
				},
				metadata: {
					types: ['dimensions'],
					maxitems: 1
				},
				visibility: true
			},		
			{
				name: 'filters',
				type: 'filters',
				metadata: {
					types: ['dimensions'],
					maxitems: -1
				}
			}
		],
		
		canColor: function (widget, panel, item) {			
			//	Handle the values panel
			if (panel.name == "value") {
				if (widget.metadata.panel("breakby").items.length > 0) {
					return false;
				} else {
					return true
				}
			}
			//	Handle the break by
			if (panel.name == "breakby") {
				return true;
			} else {
				return false;
			}
        },
		
		allocatePanel: function (widget, metadataItem) {			
			// measure
			if (prism.$jaql.isMeasure(metadataItem) && widget.metadata.panel("value").items.length === 0) {

				return "value";
			}
			// dimension
			else if (!prism.$jaql.isMeasure(metadataItem) && widget.metadata.panel("X-Axis").items.length < 3) {

				return "X-Axis";
			}
		},

		// returns true/ reason why the given item configuration is not/supported by the widget
		isSupported: function (items) {

			return this.rankMetadata(items, null, null) > -1;
		},

		// ranks the compatibility of the given metadata items with the widget
		rankMetadata: function (items, type, subtype) {

			var a = prism.$jaql.analyze(items);

			// require 1 measure, 2 dimension
			if (a.measures.length == 2 && a.dimensions.length == 1) {

				return 0;
			}

			return -1;
		},

		// populates the metadata items to the widget
		populateMetadata: function (widget, items) {

			var a = prism.$jaql.analyze(items);

			// allocating dimensions
			widget.metadata.panel("X-Axis").push(a.dimensions);
			widget.metadata.panel("value").push(a.measures);

			// allocating filters
			widget.metadata.panel("filters").push(a.filters);
		},

		// builds a jaql query from the given widget
		buildQuery: function (widget) {
	
			//  Build a jaql query object from widget metadata 
			var query = { 
				datasource: widget.datasource, 
				metadata: [] 
			};

			//  Add Dimensions
			for (i=0; i<widget.metadata.panel("X-Axis").items.length; i++) {
				var newItem = $.extend({},widget.metadata.panel("X-Axis").items[i]);
				//	Filter out nulls
				newItem.jaql.filter = {
					exclude: {
						members: ["N\\A"]
					}
				};				
				query.metadata.push(newItem);			
			}

			//  Break by
			for (i=0; i<widget.metadata.panel("breakby").items.length; i++) {
				query.metadata.push(widget.metadata.panel("breakby").items[i]);
			}

			//  Add Values
			for (i=0; i<widget.metadata.panel("value").items.length; i++) {
				query.metadata.push(widget.metadata.panel("value").items[i]);
			}

			

			//  Add Filters
			widget.metadata.panel('filters').items.forEach(function (item) {
				item = $$.object.clone(item, true);
				item.panel = "scope";
				query.metadata.push(item);
			});

			return query;
		},

		//create highcharts data structure
		processResult : function (widget, queryResult) {		
			
			//	Create array for the new query results
			var newQueryResult = [];

			//	Figure out the data series
			var metadata = queryResult.metadata(),
				hasBreakBy = widget.metadata.panel("breakby").items.length > 0,
				allData = queryResult.$$rows;
			

			//	Get the default colors palette
			var defaultColors = prism.activeDashboard.style.options.palette.colors;

			//	Function to calculate new colors
			function ColorLuminance(hex, lum) {

				// validate hex string
				hex = String(hex).replace(/[^0-9a-f]/gi, '');
				if (hex.length < 6) {
					hex = hex[0]+hex[0]+hex[1]+hex[1]+hex[2]+hex[2];
				}
				lum = lum || 0;

				// convert to decimal and change luminosity
				var rgb = "#", c, i;
				for (i = 0; i < 3; i++) {
					c = parseInt(hex.substr(i*2,2), 16);
					c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
					rgb += ("00"+c).substr(c.length);
				}

				return rgb;
			}

			//	Init the dictionary
			var data = {};

			//	Loop through all the data points, and restructure
			for (var k=0; k<allData.length; k++) {

				//	Get the data points
				var dim = allData[k][0],
					valuesIndex = hasBreakBy ? 2 : 1,
					breakbyId = hasBreakBy ? allData[k][valuesIndex-1].text : '';

				//	Loop through all measure values
				for (var l = valuesIndex; l<allData[k].length; l++) {
					
					//	Figure out an id to use
					var seriesId = breakbyId + metadata[l].jaql.dim,
						//seriesName = metadata[l].jaql.title + (hasBreakBy ? ": " + breakbyId : ""),
						seriesName = '',
						measureIndex = l - valuesIndex,
						colorAdjustmentFactor = (-0.25),
						colorAdjustmentValue = Math.ceil(measureIndex/2) * colorAdjustmentFactor;

					//	Determine a good name for the series
					if (hasBreakBy) {
						//	Using a break by, but check how many data series exist
						if (allData[k].length > valuesIndex+1) {
							//	Multiple data series
							seriesName = metadata[l].jaql.title + ": " + breakbyId;
						} else {
							//	Single data series
							seriesName = breakbyId;
						}
					} else {
						//	Not using a break by, use only the measure's title
						seriesName = metadata[l].jaql.title;
					}

					//	Get the data point
					var datapoint = allData[k][l] ? allData[k][l] : {};

					//	color darker for odds
					if (measureIndex % 2 == 1) {
						 colorAdjustmentValue *= (-1);
					}					
					var baseColor = datapoint.color ? datapoint.color : allData[k][valuesIndex-1].color,
						color = (measureIndex == 0) ? baseColor : ColorLuminance(baseColor, colorAdjustmentValue);					
						
					//	Create a data object, if it doesn't exist already
					if (!data[seriesId]){
						data[seriesId] = {
							color: color,
							type: 'line',	//	set this as default, pull from style object on render
							name: seriesName,
							data: []
						}
					}

					//	Add the data point
					var point = {
						x: dim.data,
						y: datapoint.data,
						name: dim.text,
						text: datapoint.text
					};
					data[seriesId].data.push(point);
				}
			}

			//	Transform into a valid array of series' to provide for highcharts
			for (var series in data) {
			    if (data.hasOwnProperty(series)) {			    	
			        newQueryResult.push(data[series]);
			    }
			}

			return newQueryResult;
		}
	},
	render : function (widget, event) {

		//	cancel if no results
		if (widget.queryResult.length == 0){
			return;
		}
	
		// 	Get widget element, and clear it out
		var element = $(event.element);
		element.empty();

		//	Get metadata for the chart
		var dimensionMetadata = widget.metadata.panel("X-Axis");
		var dataSeriesMetadata = widget.metadata.panel("value");
		var numSeries = dataSeriesMetadata.items.length;

		//	Get the formatting mask for the x-axis
		var xMask,
			xLevel,
			xDatatype = 'numeric';
		if (dimensionMetadata.items[0]){			
			xDatatype = dimensionMetadata.items[0].jaql.datatype;
			xLevel = dimensionMetadata.items[0].jaql.level;
			if (dimensionMetadata.items[0].format) {
				xMask = dimensionMetadata.items[0].format.mask;
			}
		}

		//	Get the formatting mask for the y-axis, just use the first (top) value series
		var yMask,
			yDatatype = 'numeric';
		if (dataSeriesMetadata.items[0]){			
			yDatatype = dataSeriesMetadata.items[0].jaql.datatype;	
			if (dataSeriesMetadata.items[0].format){
				yMask = dataSeriesMetadata.items[0].format.mask;
			}		
		}

		//	Axis formatter function
		var axisFormatter = function() {
			//	Get the datatype and init mask and value vars
			var mask,
				value,
				datatype = this.axis.options.datatype;

			//	Handle dates differently
			if (datatype == 'datetime'){
				//	Get the mask, for the datetime level
				mask = this.axis.options.mask[this.axis.options.level];
				//	Get the value
				value = new Date(this.value);
			} else {
				//	Get the mask
				mask = this.axis.options.mask;
				//	Get the value
				value = this.value;
			}
			
			//	Format the value to a text and return
			var text = prism.formatter[datatype](value,mask);			
			return text;
		}
		
		//	Create base highchart options
		var options = {
            chart: {
                zoomType: 'x'
            },            
            xAxis: {
                type: xDatatype !== 'datetime' ? 'linear' : 'datetime',
                mask: xMask,
            	datatype: xDatatype,
            	level: xLevel,
            	labels: {
            		formatter: axisFormatter
            	}
            },
            yAxis: {
            	title: {
            		text: null
            	},
				mask: yMask,
            	datatype: yDatatype,
            	labels: {
            		formatter: axisFormatter
            	}
            },
            exporting: {
            	enabled: false
            },
            legend: {
            	enabled: widget.style.showLegend
            },
            title: {
            	text: ''
            },
            plotOptions: {
            	series: {
            		step: widget.style.showStep ? 'left' : false,            		
            		marker: {
            			enabled: widget.style.showMarkers
            		}
            	}
            },
            series: widget.queryResult,
            tooltip: {
            	headerFormat: '<span style="font-size: 20px">{point.key}</span><br/>',
    			pointFormat : '<span style="color:{series.color};">{series.name}</span>: <b>{point.text}</b><br/>',
    			borderColor: 'gray',
    			shared: widget.style.groupTooltips
    		}
        };

        //	Set the X Axis title
        var xTitle = null;
        if (widget.style.showXAxisLabel) {
        	xTitle = dimensionMetadata.items[0] ? dimensionMetadata.items[0].jaql.title : null;
        }
        options.xAxis.title = {
        	text: xTitle
        };  

        //	Set the series type
        var seriesType = widget.style.chartType ? widget.style.chartType : 'line';
		for (var i=0; i<widget.queryResult.length; i++){
			options.series[i].type = seriesType;
		}	
				
		//	Draw the chart		
		element.highcharts(options);
		
	},


	destroy : function (s, e) {}
});